/**
 * ClusterTreeCellRenderer - adds tooltips and custom icons to a
 * cluster/package tree
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;

public class ClusterTreeCellRenderer extends DefaultTreeCellRenderer {

    public static final int ICON_DEFAULT = 0;
    public static final int ICON_INSTALLED = 1;
    public static final int ICON_SELECTED = 2;

    private static ImageIcon noneIcon;
    private static ImageIcon selectedIcon;
    private static ImageIcon partIcon;
    private static ImageIcon forcedIcon;
    private static ImageIcon partforcedIcon;
    private static ImageIcon selectedpartforcedIcon;

    private int icontype = ICON_DEFAULT;

    public ClusterTreeCellRenderer() {
	this(ICON_DEFAULT);
    }

    public ClusterTreeCellRenderer(int icontype) {
	this.icontype = icontype;
	if (icontype > 0) {
	    initIcons();
	}
    }

    public Component getTreeCellRendererComponent(
						  JTree tree,
						  Object value,
						  boolean sel,
						  boolean expanded,
						  boolean leaf,
						  int row,
						  boolean hasFocus) {

	super.getTreeCellRendererComponent(
					   tree, value, sel,
					   expanded, leaf, row,
					   hasFocus);
	Object o = ((DefaultMutableTreeNode) value).getUserObject();
	if (leaf) {
	    /*
	     * It would be nice if the description were more than the name.
	     */
	    SolarisPackage p = (SolarisPackage) o;
	    setToolTipText(p.getDescription());
	    if (icontype > 0) {
		if (icontype == ICON_SELECTED) {
		    if (p.getSelected()) {
			if (p.mustBeSelected()) {
			    setIcon(forcedIcon);
			} else {
			    setIcon(selectedIcon);
			}
		    } else {
			setIcon(noneIcon);
		    }
		} else {
		    if (p.exists()) {
			if (p.mustBeSelected()) {
			    setIcon(forcedIcon);
			} else {
			    setIcon(selectedIcon);
			}
		    } else {
			setIcon(noneIcon);
		    }
		}
	    }
	} else {
	    if (o instanceof PackageCluster) {
		PackageCluster pc = (PackageCluster) o;
		setToolTipText(pc.getDescription());
		if (icontype > 0) {
		    int istat = (icontype == ICON_SELECTED)
			? pc.selectedStatus() : pc.installedStatus();
		    if (istat == PackageCluster.FULLY_INSTALLED) {
			if (pc.mustBeSelected()) {
			    setIcon(forcedIcon);
			} else if (pc.mustBePartiallySelected()) {
			    setIcon(selectedpartforcedIcon);
			} else {
			    setIcon(selectedIcon);
			}
		    } else if (istat == PackageCluster.NOT_INSTALLED) {
			setIcon(noneIcon);
		    } else {
			if (pc.mustBePartiallySelected()) {
			    setIcon(partforcedIcon);
			} else {
			    setIcon(partIcon);
			}
		    }
		}
	    } else {
		setToolTipText(null);
	    }
	}

	return this;
    }

    /*
     * Absolute paths, otherwise they get resolved relative to this
     * class itself which is deep down in the hierarchy.
     */
    private void initIcons() {
	noneIcon = createImageIcon("/images/none.png");
	selectedIcon = createImageIcon("/images/selected.png");
	partIcon = createImageIcon("/images/part.png");
	forcedIcon = createImageIcon("/images/forced.png");
	partforcedIcon = createImageIcon("/images/partforced.png");
	selectedpartforcedIcon =
	    createImageIcon("/images/selectedpartforced.png");
    }

    /*
     * Based on the Java Swing tutorial examples.
     */
    private ImageIcon createImageIcon(String s) {
	java.net.URL imgURL = getClass().getResource(s);
	if (imgURL != null) {
	    return new ImageIcon(imgURL);
	} else {
	    return null;
	}
    }

}
