/**
 * ContentsFileDetail - describes a line in the contents file
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import java.util.*;

public class ContentsFileDetail {

    protected String filename;
    protected String ftype;
    protected String pclass;
    protected String owner;
    protected String group;
    protected String mode;
    // stored as String, let consumers do the conversion work
    protected String size;
    protected String cksum;
    protected String modtime;
    protected List <String> pkglist;
    // only valid for links
    protected String target;
    // major + minor only valid for devices, ignore as not used

    /**
     * Create a set of details from a line of the contents file.
     */
    public ContentsFileDetail(String s) {
	pkglist = new ArrayList <String> ();
	parseNewStyle(s);
    }

    /**
     * Parse the line. We pick the name and the type, then parse the
     * rest according to the type. From contents(4)
     *
     * ftype s: path=rpath s class package
     * ftype l: path l class package
     * ftype d: path d class mode owner group package(s)
     * ftype b: path b class major minor mode owner group package
     * ftype c: path c class major minor mode owner group package
     * ftype f: path f class mode owner group size cksum modtime package
     * ftype x: path x class mode owner group package
     * ftype v: path v class mode owner group size cksum modtime package
     * ftype e: path e class mode owner group size cksum modtime package
     */
    private void parseNewStyle(String s) {
	StringTokenizer st = new StringTokenizer(s, " ");
	filename = st.nextToken();
	ftype = st.nextToken();
	pclass = st.nextToken();
	// deal with links first
	if (ftype.equals("l") || ftype.equals("s")) {
	    // split the filename into name and link target
	    String[] ds = filename.split("=", 2);
	    filename = ds[0];
	    target = ds[1];
	    while (st.hasMoreTokens()) {
		pkglist.add(st.nextToken());
	    }
	    return;
	}
	if (ftype.equals("b") || ftype.equals("c")) {
	    // skip major and minor deviuce numbers
	    st.nextToken();
	    st.nextToken();
	}
	mode = st.nextToken();
	owner = st.nextToken();
	group = st.nextToken();
	if (ftype.equals("f") || ftype.equals("v") || ftype.equals("e")) {
	    size = st.nextToken();
	    cksum = st.nextToken();
	    modtime = st.nextToken();
	}
	// anything left is a package
	while (st.hasMoreTokens()) {
	    pkglist.add(st.nextToken());
	}
    }

    /**
     * Return the name of the file associated with this entry.
     */
    public String getName() {
	return filename;
    }

    /**
     * Return the list of names of packages that own this entry.
     */
    public List <String> getPackageNames() {
	return pkglist;
    }

    /**
     * Return the list of packages that own this entry.
     */
    public List <SolarisPackage> getPackages() {
	List <SolarisPackage> lp = new ArrayList <SolarisPackage> ();
	for (String s : pkglist) {
	    lp.add(new SolarisPackage(s));
	}
	return lp;
    }

    /**
     * Return whether this entry is shared amongst packages, determined
     * by whether the number of packages that own this entry is one
     * or more than one.
     */
    public boolean isShared() {
	return (pkglist.size() != 1);
    }

    /**
     * Return whether this entry is a directory, which means that it
     * would be of type d or x (x denotes a directory that is exclusive
     * to a package)
     */
    public boolean isDirectory() {
	return (ftype.equals("d") || ftype.equals("x"));
    }

    /**
     * Return whether this entry is a regular file, which means that it
     * would be of type f or v or e. As a result, it will have a size,
     * checksum, and modification time
     */
    public boolean isRegular() {
	return (ftype.equals("e") || ftype.equals("f") || ftype.equals("v"));
    }

    /**
     * Return whether this entry is a link to another file.
     * This is denoted by it being of type l (hard link) or
     * type s (symbolic link).
     */
    public boolean isLink() {
	return (ftype.equals("l") || ftype.equals("s"));
    }

    /**
     * Return the type of this entry as a descriptive String.
     */
    public String getDescriptiveType() {
	String t;
	if (ftype.equals("b")) {
	    t = "Block special device.";
	} else if (ftype.equals("b")) {
	    t = "Character special device.";
	} else if (ftype.equals("d")) {
	    t = "Directory.";
	} else if (ftype.equals("e")) {
	    t = "Editable file.";
	} else if (ftype.equals("f")) {
	    t = "Regular file.";
	} else if (ftype.equals("i")) {
	    t = "Information file.";
	} else if (ftype.equals("l")) {
	    t = "Hard linked file.";
	} else if (ftype.equals("p")) {
	    t = "Named pipe.";
	} else if (ftype.equals("s")) {
	    t = "Symbolic link.";
	} else if (ftype.equals("v")) {
	    t = "Volatile file.";
	} else if (ftype.equals("x")) {
	    t = "Directory exclusive to this package.";
	} else {
	    t = "Unknown.";
	}
	return t;
    }

}
