/**
 * DistributionClusterPanel
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import javax.swing.*;
import javax.swing.tree.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;

import org.jdesktop.swingworker.SwingWorker;

public class DistributionClusterPanel extends JPanel {

    private boolean showselected;

    private final JTree tree;

    private PkgList plist;
    private ClusterToc ctoc;

    private PackageInformationPanel pip;
    private DistributionSelectionPanel dsp;

    private SolarisPackage currentPackage = null;

    public DistributionClusterPanel(PkgList plist, ClusterToc ctoc,
			    boolean showselected, PackageProfile profile) {
	this.plist = plist;
	this.ctoc = ctoc;
	this.showselected = showselected;

	setLayout(new BorderLayout());

	// tree view of clusters
	DefaultMutableTreeNode topnode = new DefaultMutableTreeNode("Solaris");
	tree = new JTree(topnode);
	ClusterTree ct = new ClusterTree(topnode, ctoc);
	tree.expandRow(0);

	/*
	 * Set a renderer to do tooltips and custom icons. If showselected
	 * is true, then we're being called from the profile builder and
	 * we show the current state of package selection. Otherwise we're
	 * just displaying a distribution and the notion of a package or
	 * cluster being selected isn't valid.
	 */
	int istyle = (showselected) ? ClusterTreeCellRenderer.ICON_SELECTED
	    : ClusterTreeCellRenderer.ICON_DEFAULT;
	tree.setCellRenderer(new ClusterTreeCellRenderer(istyle));
	ToolTipManager.sharedInstance().registerComponent(tree);

	// Listen for when the selection changes.
	tree.addTreeSelectionListener(new TreeSelectionListener() {
	    public void valueChanged(TreeSelectionEvent e) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)
		    tree.getLastSelectedPathComponent();

		if (node != null) {
		    if (node.isLeaf()) {
			setPackageInfo((SolarisPackage) node.getUserObject());
		    } else {
			Object o = node.getUserObject();
			if (o instanceof String) {
			    setClusterInfo(o.toString());
			} else {
			    setClusterInfo((PackageCluster) o);
			}
		    }
		}
	    }
	});
	// end of tree listener

	// tabbed info pane on right
	pip = new PackageInformationPanel();

	// if showselected then we're in the profile builder
	// and need to show the selection panel
	if (showselected) {
	    dsp = new DistributionSelectionPanel(profile, plist, ctoc);
	    pip.showDsp(PkgResources.getString("JS.MSG.CUSTOM"), dsp);
	}

	// split pane to hold the lot
	JSplitPane psplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
		new JScrollPane(tree), pip);
	psplit.setOneTouchExpandable(true);
	psplit.setDividerLocation(180);
	add(psplit);
    }

    MouseListener mouseListener = new MouseAdapter() {
	public void mouseClicked(MouseEvent e) {
	    Cursor c = getCursor();
	    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    JList source = (JList) e.getSource();
	    setPackageInfo((SolarisPackage) source.getSelectedValue());
	    setCursor(c);
	}
    };

    private void setClusterInfo(String s) {
	currentPackage = null;
	MetaCluster mc = ctoc.getMetaCluster(s);
	if (mc != null) {
	    StringBuilder sb = new StringBuilder();
	    sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\">");
	    sb.append("<th>MetaCluster Name</th></tr>\n");
	    sb.append("<tr><td>").append(mc.getName()).append("</td></tr>\n");
	    sb.append("</table>\n");
	    pip.setInfoText(sb.toString());
	    pip.setClusterText("");
	} else {
	    pip.setInfoText(s);
	}
    }

    private void setClusterInfo(PackageCluster pc) {
	currentPackage = null;
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\">");
	sb.append("<th>Cluster Name</th></tr>\n");
	sb.append("<tr><td>").append(pc.getName()).append("</td></tr>\n");
	sb.append("</table>\n");
	pip.setInfoText(sb.toString(),
			PkgUtils.dependencyTable(pc, plist, showselected),
			revDeps(pc));
	setMetaClusterMembership(pc);
	if (showselected) {
	    dsp.showSelection(pc);
	}
    }

    // FIXME this doesn't belong here
    private String revDeps(PackageCluster pc) {
	Set <SolarisPackage> revSet = pc.getDependantSet();
	if (revSet == null) {
	    return "";
	}
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\">");
	sb.append("<th>Depending on this cluster:</th></tr>\n");
	for (SolarisPackage pkg : revSet) {
	    sb.append("<tr><td>").append(pkg.getName());
	    if (showselected && pkg.getSelected()) {
		sb.append(" (selected)");
	    }
	    sb.append("</td></tr>\n");
	}
	sb.append("</table>\n");
	return sb.toString();
    }

    /**
     * Display which metaclusters a given cluster is in.
     */
    private void setMetaClusterMembership(PackageCluster pc) {
	StringBuilder sb = new StringBuilder();
	sb.append("<table width=\"100%\">");
	sb.append("<tr bgcolor=\"#eeeeee\"><th colspan=\"2\">");
	sb.append("This cluster is part of the following Metaclusters:");
	sb.append("</th></tr>\n");
	for (MetaCluster mclust : ctoc.containingMetaClusters(pc)) {
	    sb.append("<tr><td>").append(mclust).append("</td><td>");
	    sb.append(mclust.getName()).append("</td></tr>\n");
	}
	sb.append("</table>\n");
	pip.setClusterText(sb.toString());
    }

    private void setPackageInfo(SolarisPackage pkg) {
	currentPackage = pkg;
	pip.setInfoText(PkgUtils.infoTable(pkg),
		PkgUtils.dependencyTable(pkg, plist, showselected),
		plist.revDeps(pkg.getName(), showselected));
	pip.setClusterText(PkgUtils.clusterMembership(pkg, ctoc));
	if (showselected) {
	    dsp.showSelection(pkg);
	}
    }

    public void createRevDependencies() {
	(new RevDependencyWorker()).execute();
    }

    class RevDependencyWorker extends SwingWorker <String, Object> {
	@Override
	public String doInBackground() {
	    plist.createRevDependencies();
	    return "done";
	}

	@Override
	protected void done() {
	    try {
		pip.showRevDependencies();
		if (currentPackage != null) {
		    setPackageInfo(currentPackage);
		}
	    } catch (Exception ignore) {
	    }
	}
    }

    // FIXME the tree doesn't actually expand
    public void setMetaCluster(String metacluster) {
	DefaultMutableTreeNode rootnode =
	    (DefaultMutableTreeNode) tree.getModel().getRoot();
	for (Enumeration e = rootnode.children(); e.hasMoreElements(); ) {
	    DefaultMutableTreeNode thisnode =
		(DefaultMutableTreeNode) e.nextElement();
	    if (metacluster.equals(thisnode.getUserObject())) {
		tree.expandPath(new TreePath(thisnode));
	    }
	}
    }
}
