/**
 * DistributionPackagePanel
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import uk.co.petertribble.jingle.TableSorter;

import org.jdesktop.swingworker.SwingWorker;

/**
 * Show a table inside a Scroll pane, displaying the packages installed.
 * The columns can be sorted by clicking the column headers. Selecting
 * a row causes information to appear in a tabbed area below the table.
 */
public class DistributionPackagePanel extends JPanel {

    private boolean showselected;

    private PackageInformationPanel pip;
    private DistributionSelectionPanel dsp;
    private PackageTableModel ptm;

    private SolarisPackage currentPackage = null;

    private PkgList plist;
    private ClusterToc ctoc;

    public DistributionPackagePanel(PkgList plist, ClusterToc ctoc,
			    boolean showselected, PackageProfile profile) {
	this.plist = plist;
	this.ctoc = ctoc;
	this.showselected = showselected;

	setLayout(new BorderLayout());

	JPanel jpp = new JPanel(new BorderLayout());
	ptm = new PackageTableModel(plist, showselected);
	final TableSorter sortedModel = new TableSorter(ptm);
	final JTable ptable = new JTable(sortedModel);
	jpp.add(new JScrollPane(ptable));
	sortedModel.setTableHeader(ptable.getTableHeader());

	pip = new PackageInformationPanel();

	JSplitPane psplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
		jpp, pip);
	psplit.setOneTouchExpandable(true);
	psplit.setDividerLocation(180);
	add(psplit);

	ptable.getSelectionModel().addListSelectionListener(
		new ListSelectionListener() {
	    public void valueChanged(ListSelectionEvent e) {
                if (e.getSource() instanceof DefaultListSelectionModel) {
                    if (!e.getValueIsAdjusting()) {
			int irow = ptable.getSelectedRow();
			if (irow >= 0) {
			    // sorted table, need to convert the index
			    int imod = sortedModel.modelIndex(irow);
			    SolarisPackage pkg = ptm.getPackageAtRow(imod);
			    setPackageInfo(pkg);
			}
		    }
		}
	    }
	});

	// if showselected then we're in the profile builder
	// and need to show the selection panel
	if (showselected) {
	    dsp = new DistributionSelectionPanel(profile, plist, ctoc);
	    pip.showDsp(PkgResources.getString("JS.MSG.CUSTOM"), dsp);
	}
    }

    public void setPackageInfo(SolarisPackage pkg) {
	if (pkg.exists()) {
	    currentPackage = pkg;
	    pip.setInfoText(PkgUtils.infoTable(pkg),
		    PkgUtils.dependencyTable(pkg, plist, showselected),
		    plist.revDeps(pkg.getName()));
	} else {
	    pip.setInfoText("Not installed");
	}
	pip.setClusterText(PkgUtils.clusterMembership(pkg, ctoc));
	if (showselected) {
	    dsp.showSelection(pkg);
	}
    }

    public void createRevDependencies() {
	(new RevDependencyWorker()).execute();
    }

    class RevDependencyWorker extends SwingWorker <String, Object> {
	@Override
	public String doInBackground() {
	    plist.createRevDependencies();
	    return "done";
	}

	@Override
	protected void done() {
	    try {
		pip.showRevDependencies();
		if (currentPackage != null) {
		    setPackageInfo(currentPackage);
		}
	    } catch (Exception ignore) {
	    }
	}
    }

    private void setPackagesSelected(MetaCluster mcluster) {
	for (SolarisPackage pkg : plist.getPackages()) {
	    pkg.setSelected(false);
	}
	mcluster.setSelected(true);
    }

    public void setMetaCluster(String metacluster) {
	MetaCluster mcluster = ctoc.getMetaCluster(metacluster);
	if (showselected && (mcluster != null)) {
	    setPackagesSelected(mcluster);
	}
    }
}
