/**
 * MetaCluster - describe a Solaris install metacluster
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.pkgview;

import java.util.List;
import java.util.ArrayList;

public class MetaCluster {

    private String cname;
    private String name;
    private String description;
    private List <PackageCluster> clusters;
    private List <SolarisPackage> packages;

    private boolean selected = false;

    public MetaCluster(String cname) {
	this.cname = cname;
	packages = new ArrayList <SolarisPackage> ();
	clusters = new ArrayList <PackageCluster> ();
    }

    public void setName(String name) {
	this.name = name;
    }

    public void setDescription(String description) {
	this.description = description;
    }

    public String getName() {
	return name;
    }

    public String getDescription() {
	return description;
    }

    public String toString() {
	return cname;
    }

    public String getClusterName() {
	return cname;
    }

    public void addCluster(PackageCluster pc) {
	clusters.add(pc);
    }

    public void addPackage(SolarisPackage pkg) {
	packages.add(pkg);
    }

    /**
     * Returns all the clustrs of packages that are contained
     * in this MetaCluster.
     */
    public List <PackageCluster> getClusters() {
	return clusters;
    }

    /**
     * Returns all the individual packages that are explicitly contained
     * in this MetaCluster. Does not recurse into clusters.
     */
    public List <SolarisPackage> getPackages() {
	return packages;
    }

    /**
     * Gets whether this metacluster explicitly contains the specified
     * package cluster.
     */
    public boolean containsCluster(PackageCluster pc) {
	if (clusters.contains(pc)) {
	    return true;
	}
	// check if any names match
	return containsCluster(pc.getClusterName());
    }

    /**
     * Gets whether this metacluster explicitly contains the specified
     * package cluster.
     */
    public boolean containsCluster(String pcname) {
	// check if any names match
	for (PackageCluster pkgc : clusters) {
	    if (pkgc.getClusterName().equals(pcname)) {
		return true;
	    }
	}
	return false;
    }

    /**
     * Gets whether this metacluster explicitly contains the specified
     * package.
     */
    public boolean containsPackage(SolarisPackage p) {
	if (packages.contains(p)) {
	    return true;
	}
	// check if any names match
	return containsPackage(p.getName());
    }

    /**
     * Gets whether this metacluster explicitly contains the specified
     * package.
     */
    public boolean containsPackage(String pname) {
	// check if any names match
	for (SolarisPackage pkg : packages) {
	    if (pkg.getName().equals(pname)) {
		return true;
	    }
	}
	return false;
    }

    /**
     * Gets whether this metacluster contains the specified package,
     * either explicitly or implictly via membership of a cluster.
     */
    public boolean includesPackage(SolarisPackage p) {
	if (containsPackage(p)) {
	    return true;
	}
	// check clusters
	for (PackageCluster pkgc : clusters) {
	    if (pkgc.containsPackage(p)) {
		return true;
	    }
	}
	return false;
    }

    /**
     * Gets whether this metacluster contains the specified package,
     * either explicitly or implictly via membership of a cluster.
     */
    public boolean includesPackage(String pname) {
	if (containsPackage(pname)) {
	    return true;
	}
	// check clusters
	for (PackageCluster pkgc : clusters) {
	    if (pkgc.containsPackage(pname)) {
		return true;
	    }
	}
	return false;
    }

    /**
     * Sets whether this metacluster is selected. Propagates this selection to
     * contained clusters and packages.
     */
    public void setSelected(boolean selected) {
	this.selected = selected;
	for (PackageCluster pkgc : clusters) {
	    pkgc.setSelected(selected);
	}
	for (SolarisPackage pkg : packages) {
	    pkg.setSelected(selected);
	}
    }

    /**
     * Gets whether this metacluster is selected
     */
    public boolean getSelected() {
	return selected;
    }

    /**
     * Sets whether the packages and clusters in this metacluster are
     * unselectable. Normally used only for the SUNWCmreq metacluster.
     */
    public void forceSelected() {
	for (PackageCluster pkgc : clusters) {
	    pkgc.forceSelected();
	}
	for (SolarisPackage pkg : packages) {
	    pkg.forceSelected();
	}
    }
}
