/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package uk.co.petertribble.solview.explorer;

import java.util.*;
import javax.swing.table.*;
import java.awt.event.*;
import uk.co.petertribble.jkstat.api.*;

public class FsstatTableModel extends AbstractTableModel
		implements ActionListener {

    private static final String vop_prefix = "vopstats_";

    // standard choices for columns
    // read/write
    private static String title_1 = "I/O operations";
    private static String[] names_1 = { "nread", "nwrite", "nreaddir",
		"read_bytes", "write_bytes", "readdir_bytes" };
    // create/remove
    private static String title_2 = "Create/Remove";
    private static String[] names_2 = { "ncreate", "nremove", "nmkdir",
		"nrmdir", "nlink", "nsymlink" };
    // file status
    private static String title_3 = "File access";
    private static String[] names_3 = { "nlookup", "naccess", "npathconf",
		"nreadlink" };
    // attributes - this is exactly fsstat -a
    private static String title_4 = "Attributes";
    private static String[] names_4 = { "ngetattr", "nsetattr", "ngetsecattr",
		"nsetsecattr" };
    // map - this is exactly fsstat -v
    private static String title_5 = "fsstat -v";
    private static String[] names_5 = { "nmap", "naddmap", "ndelmap",
		"ngetpage", "nputpage", "npageio" };
    private String[] columnNames;
    private String columnTitle;

    static final protected Map <String, String[]> columnMap;
    protected Vector <Kstat> allkstat;
    protected Vector <Kstat> vkstat;
    protected Vector <Kstat> oldvkstat;
    protected Vector <String> vfsname;

    protected javax.swing.Timer timer;
    protected int delay = 1000;
    private JKstat jkstat;
    private Mnttab mnttab;

    // display masks
    private int filtermask;
    // don't include anything with the ignore option
    private static final int MASK_IGNORE = 1;
    // only show those in the shown zone list
    private static final int MASK_BYZONE = 2;
    // only show those in the shown fstype list
    private static final int MASK_BYTYPE = 4;
    // don't show aggregates which have no filesystems
    private static final int MASK_MINAGGR = 8;
    // don't show any filesystem aggregates at all
    private static final int MASK_ALLAGGR = 16;
    // Vectors to hold lists of zones, fstypes to show
    private Vector <String> showzones;
    private Vector <String> showfstypes;

    static {
	columnMap = new HashMap <String, String[]> ();
	columnMap.put(title_1, names_1);
	columnMap.put(title_2, names_2);
	columnMap.put(title_3, names_3);
	columnMap.put(title_4, names_4);
	columnMap.put(title_5, names_5);
    }

    /**
     * Create a Table Model from the given Vector of fsstat kstats
     */
    public FsstatTableModel(Vector <Kstat> allkstat, int interval,
		JKstat jkstat) {
	this(allkstat, interval, jkstat, MASK_IGNORE | MASK_ALLAGGR);
    }

    /**
     * Create a Table Model from the given Vector of fsstat kstats
     */
    public FsstatTableModel(Vector <Kstat> allkstat, int interval,
		JKstat jkstat, int filtermask) {
	this.allkstat = allkstat;
	this.jkstat = jkstat;
	this.filtermask = filtermask;
	// save the kstat list for reference
	vkstat = new Vector <Kstat> (allkstat.size());
	oldvkstat = new Vector <Kstat> (vkstat.size());
	showzones = new Vector <String> ();
	showfstypes = new Vector <String> ();
	// calculate the filesystem names
	vfsname = new Vector <String> (vkstat.size());
	mnttab = new Mnttab();
	updateFilter();
	// setFsNames();
	// set up for regular updates
	delay = interval*1000;
	updateKstat();
	updateKstat();
	setNames(title_1);
	startLoop();
    }

    public void startLoop() {
	if (delay > 0) {
	    if (timer == null) {
		timer = new javax.swing.Timer(delay, this);
	    }
	    timer.start();
	}
    }

    public void stopLoop() {
	if (timer != null) {
	    timer.stop();
	}
    }

    /**
     * Set the loop delay to be the specified number of seconds.
     * If a zero or negative delay is requested, stop the updates
     * and remember the previous delay.
     */
    public void setDelay(int interval) {
	if (interval <= 0) {
	    stopLoop();
	} else {
	    delay = interval*1000;
	    if (timer != null) {
		timer.setDelay(delay);
	    }
	}
    }

    public void actionPerformed(ActionEvent e) {
	updateKstat();
    }

    // filtering
    public void showZone(String zs, boolean b) {
	if (b) {
	    if (!showzones.contains(zs)) {
		showzones.add(zs);
	    }
	} else {
	    showzones.remove(zs);
	}
    }

    public void showFstype(String fs, boolean b) {
	if (b) {
	    if (!showfstypes.contains(fs)) {
		showfstypes.add(fs);
	    }
	} else {
	    showfstypes.remove(fs);
	}
    }

    /**
     * Initializes the active lists of kstats by filtering the full list.
     * Also initializes the list of filesystem names.
     */
    public void updateFilter() {
	vkstat.clear();
	vfsname.clear();
	for (Kstat ks : allkstat) {
	    boolean doadd = true;
	    String dev2 = ks.getName().substring(vop_prefix.length());
	    String dev3 = mnttab.getFSforDevice(dev2);

	    // first check if we should ignore it
	    if (((filtermask & MASK_IGNORE) != 0) && (dev3 != null) &&
					(mnttab.getIgnore(dev3))) {
		doadd = false;
	    }

	    // check if ignoring all aggregates
	    // aggregates aren't mounted so dev3 is null
	    if (doadd && ((filtermask & MASK_ALLAGGR) != 0) && (dev3 == null)) {
		doadd = false;
	    }

	    // FIXME filter on active aggregates

	    // FIXME filter on fstype

	    // FIXME filter on zonename

	    if (doadd) {
		vkstat.add(ks);
		if (dev3 == null) {
		    vfsname.add(dev2);
		} else {
		    vfsname.add(dev3);
		}
	    }
	}
    }

    public void updateKstat() {
	// FIXME leverage KstatSet
	jkstat.chainUpdate();
	boolean listchanged = false;
	oldvkstat.clear();
	Iterator vki = vkstat.iterator();
	int i = 0;
	while (vki.hasNext()) {
	    Kstat ks = (Kstat) vki.next();
	    Kstat ks_new = jkstat.getKstat(ks);
	    if (ks_new == null) {
		vki.remove();
		listchanged = true;
	    } else {
		oldvkstat.add(ks);
		vkstat.set(i, ks_new);
		++i;
	    }
	}
	if (listchanged) {
	    setFsNames();
	    fireTableStructureChanged();
	}
	fireTableDataChanged();
    }

    public void setNames(String s) {
	columnTitle = s;
	columnNames = columnMap.get(columnTitle);
	if (columnNames == null) {
	    columnNames = names_1;
	}
	fireTableStructureChanged();
    }

    public String getCurrentTitle() {
	return columnTitle;
    }

    public Set <String> getTitles() {
	return columnMap.keySet();
    }

    public int getColumnCount() {
	// 1 extra for the device column
	return columnNames.length + 1;
    }

    public int getRowCount() {
	return vkstat.size();
    }

    public String getColumnName(int col) {
	return (col == columnNames.length) ? "Device" : columnNames[col];
    }

    public void setFsNames() {
	for (Kstat ks : vkstat) {
	    vfsname.add(getFsName(ks));
	}
    }

    private String getFsName(Kstat ks) {
	String dev2 = ks.getName().substring(vop_prefix.length());
	String dev3 = mnttab.getFSforDevice(dev2);
	return (dev3 == null) ? dev2 : dev3;
    }

    public Object getValueAt(int row, int col) {
	if (col == columnNames.length) {
	    return vfsname.get(row);
	} else {
	    // must be regular data
	    Kstat ks_old = oldvkstat.get(row);
	    Kstat ks_new = vkstat.get(row);
	    if ((ks_old == null) || (ks_new == null)) {
		return new Double(Double.NaN);
	    }
	    long snapdelta = ks_new.getSnaptime() - ks_old.getSnaptime();
	    long lold = ks_old.longData(columnNames[col]);
	    long lnew = ks_new.longData(columnNames[col]);
	    return (snapdelta == 0) ? new Double(Double.NaN)
		: new Double((lnew-lold)*1000000000.0/snapdelta);
	}
    }

    public Class getColumnClass(int c) {
	return (c ==  columnNames.length) ? String.class : Double.class;
    }

    /*
     * Don't need to implement this method unless your table's editable.
     */
    public boolean isCellEditable(int row, int col) {
	return false;
    }
}
