/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package uk.co.petertribble.solview.explorer;

import uk.co.petertribble.jkstat.api.*;
import java.util.*;
import java.text.DateFormat;

/**
 * Emulates psrinfo(1M)
 */
public class JCpuInfo {

    private JKstat jkstat;

    // for arguments
    private boolean flag_v;
    private boolean flag_p;

    // for pretty printing of times
    private DateFormat df;

    public JCpuInfo(String[] args) {
	this(new JKstat(), args);
    }

    public JCpuInfo(JKstat jkstat, String[] args) {
	parseArgs(args);
	this.jkstat = jkstat;
	KstatFilter ksf = new KstatFilter(jkstat);
	ksf.addFilter("cpu_info:::");
	Set <Kstat> kstats = ksf.getKstats();

	df = DateFormat.getDateTimeInstance(DateFormat.SHORT,
					    DateFormat.SHORT);

	do_display(kstats);
    }

    private void do_display(Set <Kstat> kstats) {
	if (flag_p && flag_v) {
	    display_vp();
	} else if (flag_v) {
	    display_v(kstats);
	} else if (flag_p) {
	    display_p(kstats);
	} else {
	    display_plain(kstats);
	}
    }

    private void display_plain(Set <Kstat> kstats) {
	for (Kstat ks : kstats) {
	    ks = jkstat.getKstat(ks);
	    Date dt = new Date(1000*ks.longData("state_begin"));
	    System.out.println(ks.getInstance() + "\t" + ks.getData("state")
					+ "   since " + df.format(dt));
	}
    }

    private void display_p(Set <Kstat> kstats) {
	System.out.println(kstats.size());
    }

    private void display_v(Set <Kstat> kstats) {
	Date dtnow = new Date();
	for (Kstat ks : kstats) {
	    ks = jkstat.getKstat(ks);
	    Date dt = new Date(1000*ks.longData("state_begin"));
	    System.out.println("Status of virtual processor "
			+ ks.getInstance() + " as of: " + df.format(dtnow));
	    System.out.println("  " + ks.getData("state") + " since "
			+ df.format(dt));
	    System.out.println("  The " + ks.getData("cpu_type") +
		" processor operates at " + ks.getData("clock_MHz") + " MHz,");
	    System.out.println("        and has an " + ks.getData("fpu_type")
					+ " floating point processor.");
	}
    }

    /*
     * A slightly different version of psrinfo -vp
     */
    private void display_vp() {
	ProcessorTree proctree = new ProcessorTree(jkstat);

	/*
	 * Now walk the tree printing out the data
	 */
	for (Long l : proctree.getChips()) {
	    String brand = "";
	    Set <Long> lcore = proctree.getCores(l);
	    System.out.println("Physical processor " + l + " has " +
				lcore.size() + " cores");
	    if (proctree.isThreaded()) {
		for (Long ll : lcore) {
		    System.out.println("    Core " + ll + " has " +
				proctree.numThreads(l, ll) + " threads");
		    for (Kstat ks : proctree.coreInfoStats(l, ll)) {
			brand = (String) ks.getData("brand");
		    }
		}
	    } else {
		for (Kstat ks : proctree.chipInfoStats(l)) {
		    brand = (String) ks.getData("brand");
		}
	    }
	    System.out.println("        " + brand);
	}
    }

    private void parseArgs(String[] args) {
	int i = 0;
	while (i < args.length) {
	    if (args[i].equals("-p")) {
		flag_p = true;
	    }
	    if (args[i].equals("-v")) {
		flag_v = true;
	    }
	    if (args[i].equals("-vp") || args[i].equals("-pv")) {
		flag_p = true;
		flag_v = true;
	    }
	    i++;
	}
    }

    public static void main(String[] args) {
	new JCpuInfo(args);
    }
}
