/**
 * SysTree - shows a hierarchical hardware view
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview.explorer;

import javax.swing.*;
import javax.swing.tree.*;
import uk.co.petertribble.jkstat.api.*;
import uk.co.petertribble.solview.SolViewResources;

public class SysTree extends JTree {

    private JKstat jkstat;
    private SolViewResources svr;

    /**
     * Display a tree of hardware itesm
     */
    public SysTree(String title) {
	this(new JKstat(), title);
    }

    /**
     * Display a tree of hardware items
     */
    public SysTree(JKstat jkstat, String title) {
	this.jkstat = jkstat;

	svr = SolViewResources.getInstance();

	DefaultMutableTreeNode root = new DefaultMutableTreeNode(title);
	buildTree(root);

	setModel(new DefaultTreeModel(root));
    }

    private void buildTree(DefaultMutableTreeNode root) {
	addProcessors(root);
	addDisks(root);
	addNetworks(root);
	addMemory(root);
	addFilesystems(root);
    }

    private void addProcessors(DefaultMutableTreeNode root) {
	/*
	 * Add a node for each processor.
	 */
	ProcessorTree proctree = new ProcessorTree(jkstat);
	SysItem hi = new SysItem(SysItem.CPU_CONTAINER);
	hi.addAttribute("ptree", proctree);
	SysTreeNode htn = new SysTreeNode(hi, svr.getString("HARD.CPU"));
	root.add(htn);

	// loop over chips
	for (Long l : proctree.getChips()) {
	    int i = (int) l.longValue();
	    SysItem hi2 = new SysItem(SysItem.CPU, i);
	    hi2.addAttribute("ptree", proctree);
	    hi2.addAttribute("chip", l);
	    SysTreeNode htnchip = new SysTreeNode(hi2, l.toString());
	    if (proctree.isMulticore()) {
		// multicore, loop over cores
		for (Long ll : proctree.getCores(l)) {
		    int ii = (int) ll.longValue();
		    SysItem hi3 = new SysItem(SysItem.CPU_CORE, ii);
		    SysTreeNode htncore = new SysTreeNode(hi3,
								ll.toString());
		    hi3.addAttribute("ptree", proctree);
		    hi3.addAttribute("chip", l);
		    hi3.addAttribute("core", ll);
		    if (proctree.isThreaded()) {
			// multithreaded, loop over threads
			for (Kstat ks : proctree.coreInfoStats(l, ll)) {
			    SysItem hi4 = new SysItem(SysItem.CPU_THREAD,
								ks.getInst());
			    hi4.addAttribute("kstat", ks);
			    hi4.addAttribute("ptree", proctree);
			    hi4.addAttribute("core", ll);
			    hi4.addAttribute("chip", l);
			    htncore.add(new SysTreeNode(hi4,
							ks.getInstance()));
			}
		    } else {
			// single thread
			// we should go round the loop exactly once
			for (Kstat ks : proctree.coreInfoStats(l, ll)) {
			    hi3.addAttribute("kstat", ks);
			}
		    }
		    htnchip.add(htncore);
		}
	    } else {
		// single core
		if (proctree.isThreaded()) {
		    // multithreaded, loop over threads
		    for (Kstat ks : proctree.chipInfoStats(l)) {
			SysItem hi3 = new SysItem(SysItem.CPU_THREAD,
							ks.getInst());
			hi3.addAttribute("kstat", ks);
			hi3.addAttribute("ptree", proctree);
			hi3.addAttribute("chip", l);
			htnchip.add(new SysTreeNode(hi3,
						ks.getInstance()));
		    }
		} else {
		    // single thread
		    for (Kstat ks : proctree.chipInfoStats(l)) {
			hi2.addAttribute("kstat", ks);
		    }
		}
	    }
	    htn.add(htnchip);
	}
    }

    private void addDisks(DefaultMutableTreeNode root) {
	SysItem hi = new SysItem(SysItem.DISK_CONTAINER);
	SysTreeNode htn = new SysTreeNode(hi, svr.getString("HARD.DISK"));
	root.add(htn);

	// parent node for metadevices
	// only added if we find any
	SysTreeNode htnmd = null;

	// detailed configuratio information for metadevices
	SVMconfig svmconfig = null;

	// FIXME handle SVM metasets

	// enumerate disks
	KstatFilter ksf = new KstatFilter(jkstat);
	ksf.setFilterClass("disk");
	KstatSet kss = new KstatSet(jkstat, ksf);
	for (Kstat ks : kss.getKstats()) {
	    if (ks.getModule().equals("md")) {
		SysItem hi2 = new SysItem(SysItem.DISK_METADEVICE);
		hi2.addAttribute("kstat", ks);
		if (svmconfig == null) {
		    svmconfig = new SVMconfig();
		}
		hi2.addAttribute("config", svmconfig);
		if (htnmd == null) {
		    SysItem him = new SysItem(SysItem.DISK_META_CONTAINER);
		    htnmd = new SysTreeNode(him, svr.getString("HARD.MD"));
		    htn.add(htnmd);
		}
		htnmd.add(new SysTreeNode(hi2, ks.getName()));
	    } else {
		SysItem hi2 = new SysItem(SysItem.DISK);
		hi2.addAttribute("kstat", ks);
		SysTreeNode htndisk = new SysTreeNode(hi2, ks.getName());
		htn.add(htndisk);
		// add partitions
		KstatFilter ksfp = new KstatFilter(jkstat);
		ksfp.setFilterClass("partition");
		ksfp.addFilter(ks.getModule() + ":" + ks.getInstance() + "::");
		KstatSet kssp = new KstatSet(jkstat, ksfp);
		for (Kstat ksp : kssp.getKstats()) {
		    SysItem hi3 = new SysItem(SysItem.DISK_PARTITION);
		    hi3.addAttribute("kstat", ksp);
		    htndisk.add(new SysTreeNode(hi3, ksp.getName()));
		}
	    }
	}
    }

    private void addNetworks(DefaultMutableTreeNode root) {
	SysItem hi = new SysItem(SysItem.NET_CONTAINER);
	SysTreeNode htn = new SysTreeNode(hi, svr.getString("HARD.NETWORK"));
	root.add(htn);

	// FIXME do we need intermediate "interfaces" node

	// enumerate networks
	KstatFilter ksf = new KstatFilter(jkstat);
	ksf.setFilterClass("net");
	ksf.addFilter(":::rbytes64");
	ksf.addNegativeFilter("::mac");
	for (Kstat ks : ksf.getKstats()) {
	    SysItem hi2 = new SysItem(SysItem.NET_INTERFACE);
	    hi2.addAttribute("kstat", ks);
	    htn.add(new SysTreeNode(hi2, ks.getName()));
	}

	// add network protocols
	hi = new SysItem(SysItem.NET_PROTOCOL);
	SysTreeNode htnp = new SysTreeNode(hi, svr.getString("HARD.NETPROTO"));
	htn.add(htnp);

	htnp.add(new SysTreeNode(new SysItem(SysItem.NET_PROTO_IP), "ip"));
	htnp.add(new SysTreeNode(new SysItem(SysItem.NET_PROTO_TCP), "tcp"));
	htnp.add(new SysTreeNode(new SysItem(SysItem.NET_PROTO_UDP), "udp"));
    }

    private void addMemory(DefaultMutableTreeNode root) {
	SysItem hi = new SysItem(SysItem.MEM_CONTAINER);
	SysTreeNode htn = new SysTreeNode(hi, svr.getString("HARD.MEMORY"));
	root.add(htn);

	htn.add(new SysTreeNode(new SysItem(SysItem.MEM_KMEM),
				svr.getString("HARD.KMEM")));

	// FIXME only if the kstats exist
	htn.add(new SysTreeNode(new SysItem(SysItem.MEM_ARCSTAT),
				svr.getString("HARD.ZMEM")));
    }

    private void addFilesystems(DefaultMutableTreeNode root) {
	SysItem hi = new SysItem(SysItem.FS_CONTAINER);
	SysTreeNode htn = new SysTreeNode(hi, svr.getString("HARD.FS"));
	root.add(htn);

	// FIXME only if the kstats exist
	htn.add(new SysTreeNode(new SysItem(SysItem.FS_FSSTAT),
				svr.getString("HARD.FSSTAT")));

	ZFSconfig zconfig = new ZFSconfig();
	// FIXME only if zfs filesystems exist
	SysTreeNode ztn = new SysTreeNode(new SysItem(SysItem.ZFS_CONTAINER),
				svr.getString("HARD.ZFS"));
	htn.add(ztn);
	for (Zfilesys zfs : zconfig.filesystems()) {
	    SysItem zitem = new SysItem(SysItem.ZFS_FS);
	    zitem.addAttribute("zfs", zfs);
	    ztn.add(new SysTreeNode(zitem, zfs.getName()));
	}
    }

}
